@@ -8,6 +8,11 @@
 /* DESCRIPTION
 /* .nf
 
+ /*
+  * System library.
+  */
+#include <string.h>
+
  /*
   * Utility library.
   */
@@ -29,9 +34,7 @@
  /*
   * Postfix TLS library.
   */
-#ifdef USE_TLS
 #include <tls.h>
-#endif
 
  /*
   * State information associated with each SMTP delivery request.
@@ -77,7 +80,7 @@ typedef struct SMTP_STATE {
     /*
      * DSN Support introduced major bloat in error processing.
      */
-    VSTRING *dsn_reason;		/* on-the-fly formatting buffer */
+    DSN_BUF *why;			/* on-the-fly formatting buffer */
 } SMTP_STATE;
 
 #define SET_NEXTHOP_STATE(state, lookup_mx, domain, port) { \
@@ -103,7 +106,6 @@ typedef struct SMTP_STATE {
 #define SMTP_FEATURE_SIZE		(1<<3)
 #define SMTP_FEATURE_STARTTLS		(1<<4)
 #define SMTP_FEATURE_AUTH		(1<<5)
-#define SMTP_FEATURE_MAYBEPIX		(1<<6)	/* PIX smtp fixup mode */
 #define SMTP_FEATURE_XFORWARD_NAME	(1<<7)
 #define SMTP_FEATURE_XFORWARD_ADDR	(1<<8)
 #define SMTP_FEATURE_XFORWARD_PROTO	(1<<9)
@@ -113,6 +115,8 @@ typedef struct SMTP_STATE {
 #define SMTP_FEATURE_RSET_REJECTED	(1<<13)	/* RSET probe rejected */
 #define SMTP_FEATURE_FROM_CACHE		(1<<14)	/* cached connection */
 #define SMTP_FEATURE_DSN		(1<<15)	/* DSN supported */
+#define SMTP_FEATURE_PIX_NO_ESMTP	(1<<16)	/* PIX smtp fixup mode */
+#define SMTP_FEATURE_PIX_DELAY_DOTCRLF	(1<<17)	/* PIX smtp fixup mode */
 
  /*
   * Features that passivate under the endpoint.
@@ -140,12 +144,14 @@ typedef struct SMTP_STATE {
  /*
   * smtp.c
   */
-extern int smtp_errno;			/* XXX can we get rid of this? */
+#define SMTP_HAS_DSN(why)	(STR((why)->status)[0] != 0)
+#define SMTP_HAS_SOFT_DSN(why)	(STR((why)->status)[0] == '4')
+#define SMTP_HAS_HARD_DSN(why)	(STR((why)->status)[0] == '5')
+#define SMTP_HAS_LOOP_DSN(why) \
+    (SMTP_HAS_DSN(why) && strcmp(STR((why)->status) + 1, ".4.6") == 0)
 
-#define SMTP_ERR_NONE	0		/* no error */
-#define SMTP_ERR_FAIL	1		/* permanent error */
-#define SMTP_ERR_RETRY	2		/* temporary error */
-#define SMTP_ERR_LOOP	3		/* mailer loop */
+#define SMTP_SET_SOFT_DSN(why)	(STR((why)->status)[0] = '4')
+#define SMTP_SET_HARD_DSN(why)	(STR((why)->status)[0] = '5')
 
 extern int smtp_host_lookup_mask;	/* host lookup methods to use */
 
@@ -157,6 +163,8 @@ extern STRING_LIST *smtp_cache_dest;	/* cached destinations */
 
 extern MAPS *smtp_ehlo_dis_maps;	/* ehlo keyword filter */
 
+extern MAPS *smtp_pix_bug_maps;		/* PIX workarounds */
+
 extern MAPS *smtp_generic_maps;		/* make internal address valid */
 extern int smtp_ext_prop_mask;		/* address externsion propagation */
 
@@ -195,6 +203,7 @@ typedef struct SMTP_SESSION {
 
     time_t  expire_time;		/* session reuse expiration time */
     int     reuse_count;		/* # of times reused (for logging) */
+    int     dead;			/* No further I/O allowed */
 
 #ifdef USE_SASL_AUTH
     char   *sasl_mechanism_list;	/* server mechanism list */
@@ -205,13 +214,16 @@ typedef struct SMTP_SESSION {
 #endif
 
     /*
-     * TLS related state.
+     * TLS related state, don't forget to initialize in session_tls_init()!
      */
 #ifdef USE_TLS
-    int     tls_use_tls;		/* can do TLS */
-    int     tls_enforce_tls;		/* must do TLS */
-    int     tls_enforce_peername;	/* cert must match */
     TLScontext_t *tls_context;		/* TLS session state */
+    char   *tls_nexthop;		/* Nexthop domain for cert checks */
+    int     tls_level;			/* TLS enforcement level */
+    int     tls_retry_plain;		/* Try plain when TLS handshake fails */
+    int     tls_protocols;		/* Acceptable SSL protocols (mask) */
+    char   *tls_cipherlist;		/* Acceptable SSL ciphers */
+    char   *tls_certmatch;		/* Certificate match patterns */
 #endif
 
     SMTP_STATE *state;			/* back link */
@@ -228,6 +240,11 @@ extern void smtp_tls_list_init(void);
 
 #endif
 
+ /*
+  * What's in a name?
+  */
+#define SMTP_HNAME(rr) (var_smtp_cname_overr ? (rr)->rname : (rr)->qname)
+
  /*
   * smtp_connect.c
   */
@@ -285,14 +302,19 @@ extern int smtp_quit(SMTP_STATE *);
   * connections and other reasons why connections cannot be cached.
   */
 #define THIS_SESSION_IS_CACHED \
-	(session->expire_time > 0)
+	(!THIS_SESSION_IS_DEAD && session->expire_time > 0)
 
 #define THIS_SESSION_IS_EXPIRED \
 	(THIS_SESSION_IS_CACHED \
 	    && session->expire_time < vstream_ftime(session->stream))
 
 #define THIS_SESSION_IS_BAD \
-	(session->expire_time < 0)
+	(!THIS_SESSION_IS_DEAD && session->expire_time < 0)
+
+#define THIS_SESSION_IS_DEAD \
+	(session->dead != 0)
+
+ /* Bring the bad news. */
 
 #define DONT_CACHE_THIS_SESSION \
 	(session->expire_time = 0)
@@ -300,9 +322,26 @@ extern int smtp_quit(SMTP_STATE *);
 #define DONT_CACHE_BAD_SESSION \
 	(session->expire_time = -1)
 
+#define DONT_USE_DEAD_SESSION \
+	(session->dead = 1)
+
+ /* Initialization. */
+
+#define USE_NEWBORN_SESSION \
+	(session->dead = 0)
+
 #define CACHE_THIS_SESSION_UNTIL(when) \
 	(session->expire_time = (when))
 
+ /*
+  * Encapsulate the following so that we don't expose details of of
+  * connection management and error handling to the SMTP protocol engine.
+  */
+#define RETRY_AS_PLAINTEXT do { \
+	session->tls_retry_plain = 1; \
+	state->misc_flags &= ~SMTP_MISC_FLAG_FINAL_SERVER; \
+    } while (0)
+
  /*
   * smtp_chat.c
   */
@@ -320,12 +359,14 @@ extern void smtp_chat_init(SMTP_SESSION *);
 extern void smtp_chat_reset(SMTP_SESSION *);
 extern void smtp_chat_notify(SMTP_SESSION *);
 
-#define SMTP_RESP_FAKE(resp, _code, _dsn, _str) \
-    ((resp)->code = (_code), \
+#define SMTP_RESP_FAKE(resp, _dsn) \
+    ((resp)->code = 0, \
      (resp)->dsn = (_dsn), \
-     (resp)->str = (_str), \
+     (resp)->str = DSN_BY_LOCAL_MTA, \
      (resp))
 
+#define DSN_BY_LOCAL_MTA	((char *) 0)	/* DSN issued by local MTA */
+
  /*
   * These operations implement a redundant mark-and-sweep algorithm that
   * explicitly accounts for the fate of every recipient. The interface is
@@ -370,7 +411,7 @@ extern void smtp_rcpt_done(SMTP_STATE *, SMTP_RESP *, RECIPIENT *);
  /*
   * smtp_trouble.c
   */
-extern int smtp_sess_fail(SMTP_STATE *, DSN_BUF *);
+extern int smtp_sess_fail(SMTP_STATE *);
 extern int PRINTFLIKE(4, 5) smtp_site_fail(SMTP_STATE *, const char *,
 				             SMTP_RESP *, const char *,...);
 extern int PRINTFLIKE(4, 5) smtp_mesg_fail(SMTP_STATE *, const char *,
@@ -399,26 +440,6 @@ extern int smtp_map11_external(VSTRING *, MAPS *, int);
 extern int smtp_map11_tree(TOK822 *, MAPS *, int);
 extern int smtp_map11_internal(VSTRING *, MAPS *, int);
 
- /*
-  * smtp_dsn.c
-  */
-extern void PRINTFLIKE(6, 7) smtp_dsn_update(DSN_BUF *, const char *,
-					             const char *,
-					             int,
-					             const char *,
-					             const char *,...);
-extern void vsmtp_dsn_update(DSN_BUF *, const char *, const char *,
-			             int, const char *,
-			             const char *, va_list);
-extern void smtp_dsn_formal(DSN_BUF *, const char *, const char *, int,
-			            const char *);
-
-#define SMTP_DSN_ASSIGN(dsn, mta, stat, resp, why) \
-    DSN_ASSIGN((dsn), (stat), DSB_DEF_ACTION, (why), DSB_DTYPE_SMTP, (resp), \
-	(mta) ? DSB_MTYPE_DNS : DSB_MTYPE_NONE, (mta))
-
-#define DSN_BY_LOCAL_MTA	((char *) 0)	/* DSN issued by local MTA */
-
  /*
   * Silly little macros.
   */
