@@ -225,7 +225,7 @@ typedef struct {
 
 void    dict_register(const char *dict_name, DICT *dict_info)
 {
-    char   *myname = "dict_register";
+    const char *myname = "dict_register";
     DICT_NODE *node;
 
     if (dict_table == 0)
@@ -267,7 +267,7 @@ static void dict_node_free(char *ptr)
 
 void    dict_unregister(const char *dict_name)
 {
-    char   *myname = "dict_unregister";
+    const char *myname = "dict_unregister";
     DICT_NODE *node;
 
     if ((node = dict_node(dict_name)) == 0)
@@ -282,7 +282,7 @@ void    dict_unregister(const char *dict_name)
 
 void    dict_update(const char *dict_name, const char *member, const char *value)
 {
-    char   *myname = "dict_update";
+    const char *myname = "dict_update";
     DICT_NODE *node;
     DICT   *dict;
 
@@ -302,7 +302,7 @@ void    dict_update(const char *dict_name, const char *member, const char *value
 
 const char *dict_lookup(const char *dict_name, const char *member)
 {
-    char   *myname = "dict_lookup";
+    const char *myname = "dict_lookup";
     DICT_NODE *node;
     DICT   *dict;
     const char *ret = 0;
@@ -325,7 +325,7 @@ const char *dict_lookup(const char *dict_name, const char *member)
 
 int     dict_delete(const char *dict_name, const char *member)
 {
-    char   *myname = "dict_delete";
+    const char *myname = "dict_delete";
     DICT_NODE *node;
     DICT   *dict;
     int     result;
@@ -350,7 +350,7 @@ int     dict_delete(const char *dict_name, const char *member)
 int     dict_sequence(const char *dict_name, const int func,
 		              const char **member, const char **value)
 {
-    char   *myname = "dict_sequence";
+    const char *myname = "dict_sequence";
     DICT_NODE *node;
     DICT   *dict;
 
@@ -485,7 +485,7 @@ void    dict_walk(DICT_WALK_ACTION action, char *ptr)
 
 const char *dict_changed_name(void)
 {
-    char   *myname = "dict_changed_name";
+    const char *myname = "dict_changed_name";
     struct stat st;
     HTABLE_INFO **ht_info_list;
     HTABLE_INFO **ht;
